
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
ACKNOWLEDGEMENTS
AddBulkDomains
addon
Addon
address1
address2
AnnoCPAN
assoicated
Auth
Autoverification
AuxBilling
bona
BUILDARGS
C11
C12
C21
C31
C32
cartesian
CheckNSStatus
Compatable
Cozens
DeleteNameServer
DisableServices
EnableServices
enom
eNom
ErrX
falsey
fide
FirstName
FQDN
FQDNs
functionalty
GetContacts
GetDNS
GetDomainInfo
GetRegLock
GetRenew
GetSubAccountPassword
GetWhoisContact
GetWPPSInfo
google
gTLD
html
ICANN
ip
marshallers
ModifyNS
nameserver
nameservers
NameSpinner
ns
ns1
OrderID
P1
P2
P3
P4
P5
params
PositiveInt
PurchaseServices
RegisterNameServer
RenewServices
reseller
RRPText
SetRegLock
SetRenew
Simões
sld
SLD
slds
SLDs
Str
TechFirstName
tld
TLD
tlds
TLDs
truthy
tv
UpdateNameServer
uk
UKFSN
v0
v1
whois
WHOIS
WPPS
xml
zipcode
Robert
Stone
drzigman
lib
Net
WWW
Contact
Domain
DomainAvailability
DomainRequest
Registration
Transfer
DomainTransfer
PhoneNumber
PrivateNameServer
Role
Command
Availability
Raw
Service
ParseDomain
Types
